// GLUT.h: interface for the GLUT class.
//
//////////////////////////////////////////////////////////////////////

#ifndef PDL_GLUTI
#define PDL_GLUTI

#ifdef WIN32
#include <windows.h>
#endif
#include <GL/GL.h>
#include "GL/glut.h"

/**
 * Classe de base du systeme de gestion des interaction avec le system et openGL.
 * 
 * Le principe de cette classe est de definire d'interface abstraite de glut pour gerer les interaction
 * avec le systeme (clavier, souris...) et l'affichage.
 * cette classe definit des methodes virtuelles pure qui correspondent au callback glut. chaque classe fille est
 * donc oblig de redefinir ses methodes. ainsi, on peut facilement cre de nouveau "systemes d'interaction"(dont les
 * plus utilis sont les differents menu) en definissant juste une classe qui herite de celle ci.
 * Pour definir quel est le menu actuellement affich, cette classe possede un membre currentGlut qui est en fait
 * le menu a utiliser. pour changer de menu, il suffit de changer currentGlut et l'ensemble du systeme d'interaction
 * basculera sur ce nouveau menu.
 * En fait en interne, il existe des fonction callback glut qui, pour chaque callback glut definie, ne font qu'appeler
 * les methode du currentGlut global.
 * Quelques methodes communes a l'ensemble des systemes d'interaction sont definit dans cette classe, ce qui lui enleve son
 * status d'interface pure.
 */
class GLUT  
{
protected:
	/**
	 * le centre X de l'ecran en pixels
	 */ 
	int mouseCenterX;

	/**
	 * le centre Y de l'ecran en pixels
	 */ 
	int mouseCenterY;

	/**
	 * le glut courant. cette variable est un pointeur de pointeur afin de pouvoir etre modifier par les sous classes (menu, game).
	 */
	GLUT **currentGlut;

	/**
	 * le temps ecoul (utilis pour la gestion du moteur de particules de la souris, et le deplacement des loutres dans le menu
	 * principal)
	 */
	int elapsedTime;

	/**
	 * le temps de la derniere frame (utilis pour calcule elapsedTime)
	 */
	int lastFrameTime;
public:
	/**
	 * constructeur par defaut (inutilis.)
	 */
	GLUT();

	/**
	 * destructeur virtuel pure
	 */
	virtual ~GLUT() = 0;

	/**
	 * mehtode virtuelle pure qui doit etre utilise pour redefinir la fonction d'affichage
	 */
	virtual void draw(void) = 0;

	/**
	 * mehtode virtuelle pure qui doit etre utilise pour redefinir la fonction de calcul (hors affichage)
	 */
	virtual void idle(void) = 0;

	/**
	 * methode virtuelle pure utilise lors d'un clic souris
	 * @param boutton: le bouton cliqu (voir doc glut)
	 * @param etat: l'etat du bouton (cliqu ou non...) (voir doc glut)
	 * @param x: position x de la souris a l'ecran (en pixels)
	 * @param y: position y de la souris a l'ecran (en pixels)
	 */
	virtual void mouseClicked(int boutton,int etat,int x,int y) = 0;

	/**
	 * methode virtuelle pure utilise lorsque la souris est bouge
	 * @param x: la position X de la souris a l'ecran (en pixels)
	 * @param y: la position Y de la souris a l'ecran (en pixels)
	 */
	virtual void mouseMoved(int x,int y) = 0;

	/**
	 * methode virtuelle pure utilise lorsque la souris est bouge et qu'un bouton de la souris est appuy
	 * @param x: la position X de la souris a l'ecran (en pixels)
	 * @param y: la position Y de la souris a l'ecran (en pixels)
	 */
	virtual void mouseMovedClicked(int x,int y) = 0;

	/**
	 * methode virtuelle pure utilise lorsqu'on presse une touche
	 * @param key: le code (unicode) de la touche appuy
	 * @param x: la position X de la souris a l'ecran (en pixels)
	 * @param y: pa position Y de la souris a l'ecran (en pixels)
	 */
	virtual void keyPressed(unsigned char key,int x,int y) = 0;

	/**
	 * methode virtuelle pure utilise lorsqu'on relache une touche
	 * @param key: le code (unicode) de la touche appuy
	 * @param x: la position X de la souris a l'ecran (en pixels)
	 * @param y: pa position Y de la souris a l'ecran (en pixels)
	 */
	virtual void keyReleased(unsigned char key,int x,int y) = 0;

	/**
	 * methode virtuelle pure utilise lorsqu'on presse une touche special (shift, fleches)
	 * @param key: le code (unicode) de la touche appuy
	 * @param x: la position X de la souris a l'ecran (en pixels)
	 * @param y: pa position Y de la souris a l'ecran (en pixels)
	 */
	virtual void specialKeyFunc(int key,int x,int y) = 0;

	/**
	 * methode virtuelle pure utilise lorsqu'on relache une touche special (shift, fleches)
	 * @param key: le code (unicode) de la touche appuy
	 * @param x: la position X de la souris a l'ecran (en pixels)
	 * @param y: pa position Y de la souris a l'ecran (en pixels)
	 */
	virtual void specialKeyUpFunc(int key,int x,int y) = 0;


	/**
	 * methode virtuelle pure utilise lors du redimmentionnement de la fenetre
	 * @param key: le code (unicode) de la touche appuy
	 * @param x: la position X de la souris a l'ecran (en pixels)
	 * @param y: pa position Y de la souris a l'ecran (en pixels)
	 */
	virtual void reshape(int largeur,int hauteur) = 0;
	
	/**
	 * methode definissant le glut courant
	 * @param glut: l'instance de GLUT qui sera utilis pour la gestion des interaction
	 */
	inline void setCurrentGlut(GLUT ** glut)
	{
		this->currentGlut = glut;
	};
	
	/**
	 * methode qui permet d'acceder au glut courant
	 * @return GLUT**: le GLUT courant
	 */
	inline GLUT ** getCurrentGlut()
	{
		return (this->currentGlut);
	};

	/**
	 * methode qui calcule le temps ecoul, utilis par les menu pour gerer la souris ou encore les loutre du menu
	 * principal
	 */
	inline void calcElapsedTime()
	{
		int tmp = glutGet(GLUT_ELAPSED_TIME);
		elapsedTime = tmp - lastFrameTime;
		lastFrameTime = tmp;
	}
};

#endif
