// Joueur.h: interface for the Joueur class.
//
//////////////////////////////////////////////////////////////////////

/**
 * documentation ralis par Michel de VERDELHAN : michel.deverdelhan@free.fr
 * le 20/10/2004
 */

#ifndef JOUEUR_H
#define JOUEUR_H

#include "Objet.h"
#include "Camera.h"
#include "RULES_Vector.h"	// Added by ClassView
#include "RULES_Texture.h"
#include "GL/glut.h"
#include "SoundManager.h"

/**
 * classe representant le jouer dans le jeu.
 * le joueur est un objet qui aussi herite de camera pour la gestion des mouvements.
 * </BR> ceci pose un probleme de multiple definition de la position dans l'espace.
 * la definition utilis est celle de la camera.
 */
class Joueur : public Objet, public  Camera  
{
private:

	/**
	 * pointeur sur la position de la lumiere utilise pour afficher l'arme du joueur
	 */
	RULES_Vector * light;

	// TODO : cre un meilleur system pour passer en mode ortho
	/**
	 * la hauteur de l'ecran en pixel (utilis pour passer en mode ortho, a revoir)
	 */
	int screenHeight;

	/**
	 * la largeur de l'ecran en pixel (utilis pour passer en mode ortho, a revoir 
	 */
	int screenWidth;

	/**
	 * la texture du crosshair
	 */
	RULES_Texture * crosshair;

	/**
	 * la rotation de la texture de tir (affic au bout de l'arme)
	 */
	float textureRotation;

	/**
	 * le temp ou le joueur a commenc a tirer, utilis pour empecher de tirer trop rapidement
	 */
	int beginFireTime;

	/**
	 * le temp ecoul depuis que le joueur a tir
	 */
	int currentFireDisplayTime;

	/**
	 * le temp d'affichage de la texture de tir
	 */
	int displayFireTime;

	/**
	 * le temp minimum entre 2 tirs differents
	 */
	int shootFrequency;

	/**
	 * la texture utilis pour le tire, c'est elle qui est affich au bout de l'arme
	 */
	RULES_Texture * fireTexture;

	/**
	 * le score du joueur
	 */
	int score;

	/**
	 * sa vitesse de deplacement
	 */
	float speed;

	/**
	 * si le joueur marche (est ce utilis ???)
	 */
	bool walking;

	/**
	 * la direction dans laquelle regarde le joueur. (doublon avec la camera ???)
	 */
	RULES_Vector direction;

public:

	/**
	 * definit la position de la lumiere qui eclaire le joueur
	 * @param lightpos: la position de la lumiere
	 */
	void setLight(RULES_Vector * lightpos);

	/**
	 * retourne la position de la lumiere
	 * @return RULES_Vector: la position de la lumiere
	 */
	RULES_Vector * getLight();

	/**
	 * definit les propriete de l'ecran utilis pour l'affichage du crosshair en mode ortho
	 * @param width: la largeur de l'ecran en pixels
	 * @param height: la heuteur de l'ecran en pixels
	 */
	void setScreenAttributs(int width, int height);

	/**
	 * defient la texture de crosshair utilis par le joueur
	 * @param tex: la texture utilis. 
	 * </BR><B>ATTENTION</B> la texture est possd par le joueur et est delet par le joueur !!!
	 */
	void setCrossHairTex(RULES_Texture * tex);

	/**
	 * methode utilis pour faire tirer le joueur, lance l'affichage et joue le son si le joueur peut tirer
	 */
	void shoot();

	/**
	 * permet de savoir si le joueur est en train de tirer
	 * @return bool: true si le joueur est en train de tirer
	 */
	bool isShooting();

	/**
	 * recupere une copie de la direction du joueur
	 * @return RULES_Vector: copie de la direction du joueur
	 */
	RULES_Vector getDirection();

	/**
	 * permet de savoir si le joueur est en train de marcher
	 * @return bool: true si le joueur marche
	 */
	bool isWalking();

	/**
	 * retourne la vitesse de deplacement du joueur
	 * @return float: la vitesse du joueur
	 */
	float getSpeed();

	/**
	 * retourne le scor du joueur
	 * @return int: le score du joueur
	 */
	int getScore();

	/**
	 * retourne la frequence de tire du joueur
	 * @return int: la frequence de tir du joueur en ms
	 */
	int getShootFrequency();

	/**
	 * retourne le temps d'affichag d'un tir
	 * @return int: le temps d'affichage d'un tir (en ms)
	 */
	int getDisplayFireTime();

	/**
	 * retourne la texture afich lors d'un tir
	 * @return RULES_Texture: la texture utilis lors d'un tir
	 */
	RULES_Texture * getFireTexture();

	/**
	 * retourne le temps ecoul depuis le dernier tir
	 * @return int: le temps ecoul en ms
	 */
	int getCurrentFireDisplayTime();

	/**
	 * definit la direction du joueur
	 * @parma direc: la direction du joueur
	 */
	void setDirection(RULES_Vector direc);

	/**
	 * definit si le joueur marche (utilis ???)
	 * @param isWalking: true => le joueur marche
	 */
	void setIsWalking(bool isWalking);

	/**
	 * definit la vitesse de deplacement du joueur
	 * @param aSpeed: la vitesse du joueur
	 */
	void setSpeed(float aSpeed);

	/**
	 * definit le score du joueur. En fait il aurait plutot fallut utiliser une methode resetScore...
	 * @param aScore: le score a attribuer au joueur
	 */ 
	void setScore(int aScore);

	/**
	 * definit la frequence de tire du joueur
	 * @param: frequency: la frequence de tir du joueur
	 */
	void setShootFrequency(int frequency);

	/**
	 * definit le temps d'affichage d'un tir du joueur
	 * @param time: le temp d'affichage.
	 */
	void setDisplayFireTime(int time);

	/**
	 * definit la texture de tir du joueur
	 * @param tex: la texture de tir a assigner au joueur
	 * </BR><B>ATTENTION<B> le joueur possede la texture et la delete... ne pas essayer de la deleter...
	 */
	void setFireTexture(RULES_Texture * tex);

	/**
	 * definit le temps courant de tir, inutile dans la pratique, mais pourrait etre utiliser pour
	 * un bonus permettant de tirer immediatement par exemple...
	 * @param time: le temp courant de tir
	 */
	void setCurrentFireDisplayTime(int time);

	/**
	 * constructeur par defaut, initialis les differentes variables du joueur
	 */
	Joueur();

	/**
	 * destructeur par defaut
	 */
	virtual ~Joueur();

	/**
	 * dessine l'arme du joueur, le crossHair...
	 * @param onlyToon: definit si on doit juste afficher le toon sans la texture, pour le blooming(glow ambient)
	 */
	void draw(bool onlyToon);

	/**
	 * definit la position du joueur
	 * @param pos: la position du joueur
	 */
	void setPosition(RULES_Vector pos);

	/**
	 * retourne la position du joueur
	 * @return RULES_Vector: un copie de la position du joueur
	 */
	RULES_Vector getPosition();

};

#endif 
